#!/usr/bin/env python2.2

##redhat-logviewer
##
## Copyright (C) 2001, 2002 Red Hat, Inc.
## Copyright (C) 2001, 2002 Tammy Fox <tfox@redhat.com>

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#Originally Written by Tammy Fox
#Created June 22, 2002

## Coding conventions
## function names have underscores (_)
## variable names are semiStudlyCapped
## class names are StudlyCapped
## class instances are semiStudlyCapped

from gtk import *
import string
import gtk
import gobject
import os

import LogViewerGui
import LogFileClass
import LogBuffer

##
## I18N
## 
import gettext
gettext.bindtextdomain ("redhat-logviewer", "/usr/share/locale")
gettext.textdomain ("redhat-logviewer")
_=gettext.gettext

blankPixbuf = None      
try:
    blankPixbuf = gtk.gdk.pixbuf_new_from_file("/usr/share/redhat-logviewer/pixmaps/blank.png")
except:
    pass
alertPixbuf = None      
try:
    alertPixbuf = gtk.gdk.pixbuf_new_from_file("/usr/share/redhat-logviewer/pixmaps/alert.png")
except:
    pass

class LogFileFrame:

    def __init__(self, frameName, description, logClassInstance):
        self.logClassInstance = logClassInstance

        #--------Create Frame -------------------------------------
        
        self.logVBox = gtk.VBox(gtk.FALSE, 6)
        self.logVBox.set_border_width(6)
        self.logFrame = gtk.Frame(frameName)
        self.logFrame.set_shadow_type(gtk.SHADOW_ETCHED_IN)
        self.logFrame.set_border_width(6)        

        self.logFrame.add(self.logVBox)

        self.descriptionHBox = gtk.HBox(gtk.FALSE, 6)
        self.descriptionHBox.set_border_width(2)

        icon = None      
        try:
            icon = gtk.gdk.pixbuf_new_from_file("/usr/share/pixmaps/redhat-logviewer.png")
        except:
            pass

        image = gtk.Image()
        if icon:
            image.set_from_pixbuf(icon)

        self.descriptionHBox.pack_start(image)
        label = gtk.Label(description)
        label.set_line_wrap(gtk.TRUE)
        label.set_size_request(350, -1)
        self.descriptionHBox.pack_start(label, gtk.TRUE, gtk.TRUE, 6)

        self.logVBox.pack_start(self.descriptionHBox, gtk.FALSE)

        self.logScrollwindow = gtk.ScrolledWindow()
        self.logScrollwindow.set_policy (POLICY_AUTOMATIC, POLICY_AUTOMATIC)
        self.logScrollwindow.set_shadow_type (SHADOW_ETCHED_IN)
        
        self.logView = gtk.TextView()
        if logClassInstance == "SYSLOG":
            self.logView.set_buffer(LogViewerGui.sysClass.buffer.get_buffer())
        elif logClassInstance == "SECURELOG":
            self.logView.set_buffer(LogViewerGui.secureClass.buffer.get_buffer())
        elif logClassInstance == "MAILLOG":
            self.logView.set_buffer(LogViewerGui.mailClass.buffer.get_buffer())
        elif logClassInstance == "CRONLOG":
            self.logView.set_buffer(LogViewerGui.cronClass.buffer.get_buffer())
        elif logClassInstance == "NEWSLOG":
            self.logView.set_buffer(LogViewerGui.newsClass.buffer.get_buffer())
        elif logClassInstance == "BOOTLOG":
            self.logView.set_buffer(LogViewerGui.bootClass.buffer.get_buffer())
        elif logClassInstance == "CIPELOG":
            self.logView.set_buffer(LogViewerGui.cipeClass.buffer.get_buffer())
        elif logClassInstance == "DMESGLOG":
            self.logView.set_buffer(LogViewerGui.dmesgClass.buffer.get_buffer())
        elif logClassInstance == "HTTPDACCESSLOG":
            self.logView.set_buffer(LogViewerGui.httpdaccessClass.buffer.get_buffer())
        elif logClassInstance == "HTTPDERRORLOG":
            self.logView.set_buffer(LogViewerGui.httpderrorClass.buffer.get_buffer())
        elif logClassInstance == "MYSQLLOG":
            self.logView.set_buffer(LogViewerGui.mysqlClass.buffer.get_buffer())
        elif logClassInstance == "RPMLOG":
            self.logView.set_buffer(LogViewerGui.rpmClass.buffer.get_buffer())
        elif logClassInstance == "TUXLOG":
            self.logView.set_buffer(LogViewerGui.tuxClass.buffer.get_buffer())
        elif logClassInstance == "UP2DATELOG":
            self.logView.set_buffer(LogViewerGui.up2dateClass.buffer.get_buffer())
        elif logClassInstance == "FTPLOG":
            self.logView.set_buffer(LogViewerGui.ftpClass.buffer.get_buffer())
        elif logClassInstance == "XFREE86LOG":
            self.logView.set_buffer(LogViewerGui.xfree86Class.buffer.get_buffer())                                    
            
        self.logView.set_editable(gtk.FALSE)
        self.logView.set_cursor_visible(gtk.FALSE)
        
        self.logScrollwindow.add(self.logView)
        self.logVBox.pack_start(self.logScrollwindow)        

        self.searchHBox = gtk.HBox(gtk.FALSE, 6)
        self.searchLabel = gtk.Label(_("Filter for: "))
        self.searchHBox.pack_start(self.searchLabel)
        self.searchEntry = gtk.Entry()
        self.searchEntry.connect ("activate", self.on_search_clicked)
        self.searchHBox.pack_start(self.searchEntry, gtk.TRUE, gtk.TRUE)
        self.searchButton = self.button_with_pixbuf("/usr/share/redhat-logviewer/pixmaps/search-icon.png", _("Filter"))
        self.searchButton.connect("clicked", self.on_search_clicked)
        self.searchHBox.pack_start(self.searchButton)        
        self.logVBox.pack_start(self.searchHBox, gtk.FALSE)        

        self.resetButton = self.button_with_stock(gtk.STOCK_REFRESH, _("Reset"))
        self.resetButton.connect("clicked", self.on_reset_clicked)
        self.searchHBox.pack_start(self.resetButton, gtk.TRUE, gtk.TRUE)        

    def get_frame(self):
        return self.logFrame

    def set_buffer(self, buffer):
        self.logView.set_buffer(buffer)

    def button_with_pixbuf (self, pixbuf, labeltext):
        button = gtk.Button ()
        label = gtk.Label (labeltext)
        hbox = gtk.HBox (gtk.FALSE, 2)
        align = gtk.Alignment (0.5, 0.5, 0.0, 0.0)
        icon = None      
        try:
            icon = gtk.gdk.pixbuf_new_from_file(pixbuf)
        except:
            pass

        image = gtk.Image()
        if icon:
            image.set_from_pixbuf(icon)        
            hbox.pack_start (image)
        
        hbox.pack_end (label)
        button.add (align)
        align.add (hbox)
        button.show_all ()
        return button

    def button_with_stock (self, stock_icon, labeltext):
        button = gtk.Button ()
        label = gtk.Label (labeltext)
        hbox = gtk.HBox (gtk.FALSE, 2)
        align = gtk.Alignment (0.5, 0.5, 0.0, 0.0)
        image = gtk.Image()
        if stock_icon:
            image.set_from_stock(stock_icon, 2)
            hbox.pack_start (image)
        
        hbox.pack_end (label)
        button.add (align)
        align.add (hbox)
        button.show_all ()
        return button

## Search button callback

    def on_search_clicked(self, *args):
        ###open messages list
        ###break into readlines
        ###search for entry text
        ###if search success: write to buffer + reset text view buffer
        ###if search failed: set status bar to Search Failed.

        LogViewerGui.set_status(_("Searching file..."))

        searchResultsBuffer = LogBuffer.LogBuffer()
        success = 0
        temp_messages = None
        if self.searchEntry.get_text().strip() != "":
            searchString = self.searchEntry.get_text().strip()
            if self.logClassInstance == "SYSLOG":
                temp_messages = LogViewerGui.sysClass.get_messages()
            elif self.logClassInstance == "SECURELOG":
                temp_messages = LogViewerGui.secureClass.get_messages()
            elif self.logClassInstance == "MAILLOG":
                temp_messages = LogViewerGui.mailClass.get_messages()
            elif self.logClassInstance == "CRONLOG":
                temp_messages = LogViewerGui.cronClass.get_messages()
            elif self.logClassInstance == "NEWSLOG":
                temp_messages = LogViewerGui.newsClass.get_messages()
            elif self.logClassInstance == "BOOTLOG":
                temp_messages = LogViewerGui.bootClass.get_messages()
            elif self.logClassInstance == "CIPELOG":
                temp_messages = LogViewerGui.cipeClass.get_messages()
            elif self.logClassInstance == "DMESGLOG":
                temp_messages = LogViewerGui.dmesgClass.get_messages()
            elif self.logClassInstance == "HTTPDACCESSLOG":
                temp_messages = LogViewerGui.httpdaccessClass.get_messages()
            elif self.logClassInstance == "HTTPDERRORLOG":
                temp_messages = LogViewerGui.httpderrorClass.get_messages()
            elif self.logClassInstance == "MYSQLLOG":
                temp_messages = LogViewerGui.mysqlClass.get_messages()
            elif self.logClassInstance == "RPMLOG":
                temp_messages = LogViewerGui.rpmClass.get_messages()
            elif self.logClassInstance == "TUXLOG":
                temp_messages = LogViewerGui.tuxClass.get_messages()
            elif self.logClassInstance == "UP2DATELOG":
                temp_messages = LogViewerGui.up2dateClass.get_messages()
            elif self.logClassInstance == "FTPLOG":
                temp_messages = LogViewerGui.ftpClass.get_messages()
            elif self.logClassInstance == "XFREE86LOG":
                temp_messages = LogViewerGui.xfree86Class.get_messages()
        else:
            LogViewerGui.set_status(_("Please enter a search string."))

        if temp_messages:
            iter = searchResultsBuffer.get_iter_at_offset (0)
            for line in temp_messages:
                #insert log file line into buffer if matches search text
                if string.find(line, searchString.strip()) > -1:
                    alert = 0
                    for word in LogViewerGui.alertWords:
                        if (string.find(line.lower(), word.lower()) > -1):
                            searchResultsBuffer.insert_pixbuf(iter, alertPixbuf)
                            alert = 1
                            break
                    if alert == 0:
                        searchResultsBuffer.insert_pixbuf(iter, blankPixbuf)
                    searchResultsBuffer.insert_into_buffer_at_offset(iter, line)
                    # set status that search was successful    
                    success = 1

            #if search is successful
            if success == 1:
                if self.logClassInstance == "SYSLOG":
                    LogViewerGui.sysClass.searchBuffer = searchResultsBuffer
                    self.logView.set_buffer(LogViewerGui.sysClass.searchBuffer.get_buffer())
                elif self.logClassInstance == "SECURELOG":
                    LogViewerGui.secureClass.searchBuffer = searchResultsBuffer
                    self.logView.set_buffer(LogViewerGui.secureClass.searchBuffer.get_buffer())
                elif self.logClassInstance == "MAILLOG":
                    LogViewerGui.mailClass.searchBuffer = searchResultsBuffer
                    self.logView.set_buffer(LogViewerGui.mailClass.searchBuffer.get_buffer())
                elif self.logClassInstance == "CRONLOG":
                    LogViewerGui.cronClass.searchBuffer = searchResultsBuffer
                    self.logView.set_buffer(LogViewerGui.cronClass.searchBuffer.get_buffer())
                elif self.logClassInstance == "NEWSLOG":
                    LogViewerGui.newsClass.searchBuffer = searchResultsBuffer
                    self.logView.set_buffer(LogViewerGui.newsClass.searchBuffer.get_buffer())
                elif self.logClassInstance == "BOOTLOG":
                    LogViewerGui.bootClass.searchBuffer = searchResultsBuffer
                    self.logView.set_buffer(LogViewerGui.bootClass.searchBuffer.get_buffer())
                elif self.logClassInstance == "CIPELOG":
                    LogViewerGui.cipeClass.searchBuffer = searchResultsBuffer
                    self.logView.set_buffer(LogViewerGui.cipeClass.searchBuffer.get_buffer())
                elif self.logClassInstance == "DMESGLOG":
                    LogViewerGui.dmesgClass.searchBuffer = searchResultsBuffer
                    self.logView.set_buffer(LogViewerGui.dmesgClass.searchBuffer.get_buffer())
                elif self.logClassInstance == "HTTPDACCESSLOG":
                    LogViewerGui.httpdaccessClass.searchBuffer = searchResultsBuffer
                    self.logView.set_buffer(LogViewerGui.httpdaccessClass.searchBuffer.get_buffer())
                elif self.logClassInstance == "HTTPDERRORLOG":
                    LogViewerGui.httpderrorClass.searchBuffer = searchResultsBuffer
                    self.logView.set_buffer(LogViewerGui.httpderrorClass.searchBuffer.get_buffer())
                elif self.logClassInstance == "MYSQLLOG":
                    LogViewerGui.mysqlClass.searchBuffer = searchResultsBuffer
                    self.logView.set_buffer(LogViewerGui.mysqlClass.searchBuffer.get_buffer())
                elif self.logClassInstance == "RPMLOG":
                    LogViewerGui.rpmClass.searchBuffer = searchResultsBuffer
                    self.logView.set_buffer(LogViewerGui.rpmClass.searchBuffer.get_buffer())
                elif self.logClassInstance == "TUXLOG":
                    LogViewerGui.tuxClass.searchBuffer = searchResultsBuffer
                    self.logView.set_buffer(LogViewerGui.tuxClass.searchBuffer.get_buffer())
                elif self.logClassInstance == "UP2DATELOG":
                    LogViewerGui.up2dateClass.searchBuffer = searchResultsBuffer
                    self.logView.set_buffer(LogViewerGui.up2dateClass.searchBuffer.get_buffer())
                elif self.logClassInstance == "FTPLOG":
                    LogViewerGui.ftpClass.searchBuffer = searchResultsBuffer
                    self.logView.set_buffer(LogViewerGui.ftpClass.searchBuffer.get_buffer())
                elif self.logClassInstance == "XFREE86LOG":
                    LogViewerGui.xfree86Class.searchBuffer = searchResultsBuffer
                    self.logView.set_buffer(LogViewerGui.xfree86Class.searchBuffer.get_buffer())

                LogViewerGui.set_status(_("Search results for ") + searchString + _(" displayed."))

            else:
                LogViewerGui.set_status(searchString + _(" not found."))

    def on_reset_clicked(self, *args):
        LogViewerGui.set_status("Reseting log file...")
        if self.logClassInstance == "SYSLOG":
            if LogViewerGui.sysClass.get_messages():
                self.logView.set_buffer(LogViewerGui.sysClass.buffer.get_buffer())
            LogViewerGui.sysClass.auto_refresh()
        elif self.logClassInstance == "SECURELOG":
            if LogViewerGui.secureClass.get_messages():
                self.logView.set_buffer(LogViewerGui.secureClass.buffer.get_buffer())
            LogViewerGui.secureClass.auto_refresh()
        elif self.logClassInstance == "MAILLOG":
            if LogViewerGui.mailClass.get_messages():
                self.logView.set_buffer(LogViewerGui.mailClass.buffer.get_buffer())
            LogViewerGui.mailClass.auto_refresh()
        elif self.logClassInstance == "CRONLOG":
            if LogViewerGui.cronClass.get_messages():
                self.logView.set_buffer(LogViewerGui.cronClass.buffer.get_buffer())
            LogViewerGui.cronClass.auto_refresh()
        elif self.logClassInstance == "NEWSLOG":
            if LogViewerGui.newsClass.get_messages():
                self.logView.set_buffer(LogViewerGui.newsClass.buffer.get_buffer())
            LogViewerGui.newsClass.auto_refresh()
        elif self.logClassInstance == "BOOTLOG":
            if LogViewerGui.bootClass.get_messages():
                self.logView.set_buffer(LogViewerGui.bootClass.buffer.get_buffer())
            LogViewerGui.bootClass.auto_refresh()
        elif self.logClassInstance == "CIPELOG":
            if LogViewerGui.cipeClass.get_messages():
                self.logView.set_buffer(LogViewerGui.cipeClass.buffer.get_buffer())
            LogViewerGui.cipeClass.auto_refresh()
        elif self.logClassInstance == "DMESGLOG":
            if LogViewerGui.dmesgClass.get_messages():
                self.logView.set_buffer(LogViewerGui.dmesgClass.buffer.get_buffer())
            LogViewerGui.dmesgClass.auto_refresh()
        elif self.logClassInstance == "HTTPDACCESSLOG":
            if LogViewerGui.httpdaccessClass.get_messages():
                self.logView.set_buffer(LogViewerGui.httpdaccessClass.buffer.get_buffer())
            LogViewerGui.httpdaccessClass.auto_refresh()
        elif self.logClassInstance == "HTTPDERRORLOG":
            if LogViewerGui.httpderrorClass.get_messages():
                self.logView.set_buffer(LogViewerGui.httpderrorClass.buffer.get_buffer())
            LogViewerGui.httpderrorClass.auto_refresh()
        elif self.logClassInstance == "MYSQLLOG":
            if LogViewerGui.mysqlClass.get_messages():
                self.logView.set_buffer(LogViewerGui.mysqlClass.buffer.get_buffer())
            LogViewerGui.mysqlClass.auto_refresh()
        elif self.logClassInstance == "RPMLOG":
            if LogViewerGui.rpmClass.get_messages():
                self.logView.set_buffer(LogViewerGui.rpmClass.buffer.get_buffer())
            LogViewerGui.rpmClass.auto_refresh()
        elif self.logClassInstance == "TUXLOG":
            if LogViewerGui.tuxClass.get_messages():
                self.logView.set_buffer(LogViewerGui.tuxClass.buffer.get_buffer())
            LogViewerGui.tuxClass.auto_refresh()
        elif self.logClassInstance == "UP2DATELOG":
            if LogViewerGui.up2dateClass.get_messages():
                self.logView.set_buffer(LogViewerGui.up2dateClass.buffer.get_buffer())
            LogViewerGui.up2dateClass.auto_refresh()
        elif self.logClassInstance == "FTPLOG":
            if LogViewerGui.ftpClass.get_messages():
                self.logView.set_buffer(LogViewerGui.ftpClass.buffer.get_buffer())
            LogViewerGui.ftpClass.auto_refresh()
        elif self.logClassInstance == "XFREE86LOG":
            if LogViewerGui.xfree86Class.get_messages():
                self.logView.set_buffer(LogViewerGui.xfree86Class.buffer.get_buffer())
            LogViewerGui.xfree86Class.auto_refresh()

        LogViewerGui.set_status(_("Contents reset."))
