#!/usr/bin/env python2.2

##redhat-logviewer
##
## Copyright (C) 2001, 2002 Red Hat, Inc.
## Copyright (C) 2001, 2002 Tammy Fox <tfox@redhat.com>

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#Originally Written by Tammy Fox
#Created December 12, 2001
#Last Modified June 23, 2002

## Coding conventions
## function names have underscores (_)
## variable names are semiStudlyCapped
## class names are StudlyCapped
## class instances are semiStudlyCapped

from gtk import *
import string
import sys
import gtk
import signal
import gobject
import Preferences
import os

import LogFileClass
import LogFileFrame
import LogBuffer

##
## I18N
## 
import gettext
gettext.bindtextdomain ("redhat-logviewer", "/usr/share/locale")
gettext.textdomain ("redhat-logviewer")
_=gettext.gettext

##---------------------------------------------------------------

##
## Read in preferences file
##

try:
    prefsFile = open("/etc/sysconfig/redhat-logviewer", "r")
    prefs = prefsFile.readlines()
    prefsFile.close()
except:
    #FIXME: show message dialog that can't open pref file
    print "Preferences file not found."
    
logDict = {}
alertWords = ["failed","denied","rejected","oops","segfault","segmentation"]
for line in prefs:
    line = string.strip(line)
    if line != "" and line[0]!="#":
        preference, values = string.split(line,"=")
        preference = string.strip(preference)
        if preference == "REFRESHRATE":
            RefreshRateString = string.strip(values)
            RefreshRate = int(RefreshRateString)*1000
##         elif preference == "SHOWDATE":
##             month,day,year,hour,minute = string.split(values,",")
##             month = string.strip(month)
##             day = string.strip(day)
##             year = string.strip(year)
##             hour = string.strip(hour)
##             minute = string.strip(minute)
        elif preference == "ALERTWORDS":
            alertWords = string.split(values,",")
        else:
            logDict[preference] = values

# Don't refresh faster than 30 seconds
if RefreshRate < 30000:
    RefreshRate = 30000

##---------------------------------------------------------------

##
## Common strings
##

systemLogString = _("System Log")
secureLogString = _("Security Log")
mailLogString = _("Mail Log")
cronLogString = _("Cron Log")
newsLogString = _("News Log")
bootLogString = _("Boot Log")
cipeLogString = _("CIPE Log")
dmesgLogString = _("Kernel Startup Log")
httpdaccessLogString = _("Apache Access Log")
httpderrorLogString = _("Apache Error Log")
mysqlLogString = _("MySQL Server Log")
rpmLogString = _("RPM Packages")
tuxLogString = _("Content Accelerator Log")
up2dateLogString = _("Update Agent Log")
ftpLogString = _("FTP Log")
xfree86LogString = _("XFree86 Log")

##---------------------------------------------------------------

##
## Log Class Instances
##
        
sysClass = LogFileClass.LogFileClass("SYSLOG")
secureClass = LogFileClass.LogFileClass("SECURELOG")
mailClass = LogFileClass.LogFileClass("MAILLOG")
cronClass = LogFileClass.LogFileClass("CRONLOG")
newsClass = LogFileClass.LogFileClass("NEWSLOG")
bootClass = LogFileClass.LogFileClass("BOOTLOG")
cipeClass = LogFileClass.LogFileClass("CIPELOG")
dmesgClass = LogFileClass.LogFileClass("DMESGLOG")
httpdaccessClass = LogFileClass.LogFileClass("HTTPDACCESSLOG")
httpderrorClass = LogFileClass.LogFileClass("HTTPDERRORLOG")
mysqlClass = LogFileClass.LogFileClass("MYSQLLOG")
rpmClass = LogFileClass.LogFileClass("RPMLOG")
tuxClass = LogFileClass.LogFileClass("TUXLOG")
up2dateClass = LogFileClass.LogFileClass("UP2DATELOG")
ftpClass = LogFileClass.LogFileClass("FTPLOG")
xfree86Class = LogFileClass.LogFileClass("XFREE86LOG")              

##---------------------------------------------------------------

##
## Shared ListStore with descriptive string, file location, varname, location changed (0=FALSE), notebook page number
##

logFilesListStore = gtk.ListStore(gobject.TYPE_STRING, gobject.TYPE_STRING, gobject.TYPE_STRING, gobject.TYPE_INT, gobject.TYPE_INT)

##---------------------------------------------------------------

##
## left-hand side TreeView with descriptive strings as items
##

logFileView = gtk.TreeView(logFilesListStore)
#logFileView.set_rules_hint(gtk.TRUE)

##---------------------------------------------------------------

##
## status bar set by multiple classes
##

statusBar = gtk.Statusbar()

##
## Set Statusbar function
##

def set_status(text):
    statusBar.pop(0)
    statusBar.push(0, text)

##
## Icon for windows
##

iconPixbuf = None      
try:
    iconPixbuf = gtk.gdk.pixbuf_new_from_file("/usr/share/pixmaps/redhat-logviewer.png")
except:
    pass

##---------------------------------------------------------------
##---------------------------------------------------------------

##
## Main Class
##

class LogViewerGui:

## Destroy function for toplevel window

    def destroy(self, args):
        gtk.mainquit()

## Init

    def __init__ (self,width,height):
        self.toplevel = gtk.Window()
        self.toplevel.connect ("destroy", self.destroy)
        self.toplevel.set_title(_("System Logs"))
        self.toplevel.set_size_request(width, height)
        self.toplevel.set_position(gtk.WIN_POS_CENTER_ON_PARENT)
        self.main_vbox = gtk.VBox(gtk.FALSE, 0)
        self.toplevel.add(self.main_vbox)
        self.toplevel.set_icon(iconPixbuf)

	commentString = _("Examine system log files")

        #--------Create Menubar------------------------------------
        accel_group = gtk.AccelGroup()
        item_fac = gtk.ItemFactory(gtk.MenuBar, "<main>", accel_group)
        self.toplevel.add_accel_group(accel_group)
        item_fac.create_items([
            ((_('/_File')),                 None,            None,                  0, '<Branch>'),
            ((_('/_File/_Refresh Now')),    '<control>R',    self.auto_refresh,     6, '<StockItem>', gtk.STOCK_REFRESH),
            ((_('/_File/_Quit')),            '<control>Q',    self.on_exit_selected, 6, '<StockItem>', gtk.STOCK_QUIT),
            ((_('/_Edit')),                 None,            None,                  0, '<Branch>'),
            ((_('/_Edit/_Preferences...')), None,            self.on_preferences_selected, 6, '<StockItem>', gtk.STOCK_PREFERENCES),
            ((_('/_Help')),                 None,            None,                  0, '<Branch>'),
            ((_('/_Help/_Contents')),  None,            self.on_manual_selected, 1, '<StockItem>', gtk.STOCK_HELP),
            ((_('/_Help/_About')),          None,            self.on_about_selected, 2, ''),
            ])
        self.menuBar = item_fac.get_widget('<main>')

        self.main_vbox.pack_start(self.menuBar, gtk.FALSE)

        #--------left list store of log file types------------------
        self.main_hpane = gtk.HPaned()
        self.main_vbox.pack_start(self.main_hpane)

        #--------status bar-----------------------------------------
        self.main_vbox.pack_start(statusBar, gtk.FALSE)

        #--------Notebook that displays log files and search box----
        self.logFileNotebook = gtk.Notebook()
        self.logFileNotebook.set_show_tabs(gtk.FALSE)

        #--------populate liststore with log file names----------------
        #--------add frame instances to notebook-----------------------        

        #populate list store with log location preferences
        keys_list = logDict.keys()
        keys_list.sort()
        self.pgnum = 0
        #set item 0 of logFilesListStore to descriptive string displayed on left side
        #add frame instance to notebook in same sort order so the notebook pg number is in sort order

        #set item 1 of logFilesListStore to file location
        #set item 2 of logFilesListStore to varname
        #set item 3 of logFilesListStore to location changed (0=FALSE)
        #set item 4 of logFilesListStore to notebook page number

        for key in keys_list:
            if key == "SYSLOG" and sysClass.get_file_not_found() == 0:
                self.systemLogFrame = LogFileFrame.LogFileFrame(systemLogString, _("This is the system log file."), "SYSLOG")
                iter = logFilesListStore.append()
                self.logFileNotebook.append_page(self.systemLogFrame.get_frame(), gtk.Label(""))
                self.set_liststore_values(systemLogString, key, iter)
            elif key == "SECURELOG" and secureClass.get_file_not_found() == 0:
                self.secureLogFrame = LogFileFrame.LogFileFrame(secureLogString, _("This log file contains security messages."), "SECURELOG")
                iter = logFilesListStore.append()
                self.logFileNotebook.append_page(self.secureLogFrame.get_frame(), gtk.Label(""))
                self.set_liststore_values(secureLogString, key, iter)
            elif key == "MAILLOG" and mailClass.get_file_not_found() == 0:
                self.mailLogFrame = LogFileFrame.LogFileFrame(mailLogString, _("This log file records the mail server transactions."), "MAILLOG")
                iter = logFilesListStore.append()
                self.logFileNotebook.append_page(self.mailLogFrame.get_frame(), gtk.Label(""))
                self.set_liststore_values(mailLogString, key, iter)
            elif key == "CRONLOG" and cronClass.get_file_not_found() == 0:
                self.cronLogFrame = LogFileFrame.LogFileFrame(cronLogString, _("This log file records when cron tasks are executed."), "CRONLOG")
                iter = logFilesListStore.append()
                self.logFileNotebook.append_page(self.cronLogFrame.get_frame(), gtk.Label(""))
                self.set_liststore_values(cronLogString, key, iter)
            elif key == "NEWSLOG" and newsClass.get_file_not_found() == 0:
                self.newsLogFrame = LogFileFrame.LogFileFrame(newsLogString, _("This log file contains messages from the news server."), "NEWSLOG")
                iter = logFilesListStore.append()
                self.logFileNotebook.append_page(self.newsLogFrame.get_frame(), gtk.Label(""))
                self.set_liststore_values(newsLogString, key, iter)
            elif key == "BOOTLOG" and bootClass.get_file_not_found() == 0:
                self.bootLogFrame = LogFileFrame.LogFileFrame(bootLogString, _("This log file contains the boot messages."), "BOOTLOG")
                iter = logFilesListStore.append()
                self.logFileNotebook.append_page(self.bootLogFrame.get_frame(), gtk.Label(""))
                self.set_liststore_values(bootLogString, key, iter)
            elif key == "CIPELOG" and cipeClass.get_file_not_found() == 0:
                self.cipeLogFrame = LogFileFrame.LogFileFrame(cipeLogString, _("This log file displays messages for the CIPE interface."), "CIPELOG")
                iter = logFilesListStore.append()
                self.logFileNotebook.append_page(self.cipeLogFrame.get_frame(), gtk.Label(""))
                self.set_liststore_values(cipeLogString, key, iter)
            elif key == "DMESGLOG" and dmesgClass.get_file_not_found() == 0:
                self.dmesgLogFrame = LogFileFrame.LogFileFrame(dmesgLogString, _("This log file contains the startup messages from the kernel (dmesg)."), "DMESGLOG")
                iter = logFilesListStore.append()
                self.logFileNotebook.append_page(self.dmesgLogFrame.get_frame(), gtk.Label(""))
                self.set_liststore_values(dmesgLogString, key, iter)
            elif key == "HTTPDACCESSLOG" and httpdaccessClass.get_file_not_found() == 0:
                self.httpdaccessLogFrame = LogFileFrame.LogFileFrame(httpdaccessLogString, _("The Apache access log records the transfer of data from the server to the client."), "HTTPDACCESSLOG")
                iter = logFilesListStore.append()
                self.logFileNotebook.append_page(self.httpdaccessLogFrame.get_frame(), gtk.Label(""))
                self.set_liststore_values(httpdaccessLogString, key, iter)
            elif key == "HTTPDERRORLOG" and httpderrorClass.get_file_not_found() == 0:
                self.httpderrorLogFrame = LogFileFrame.LogFileFrame(httpderrorLogString, _("The Apache error log records errors during server and client transactions."), "HTTPDERRORLOG")
                iter = logFilesListStore.append()
                self.logFileNotebook.append_page(self.httpderrorLogFrame.get_frame(), gtk.Label(""))
                self.set_liststore_values(httpderrorLogString, key, iter)
            elif key == "MYSQLLOG" and mysqlClass.get_file_not_found() == 0:
                self.mysqlLogFrame = LogFileFrame.LogFileFrame(mysqlLogString, _("This log file contains messages for the mySQL Server."), "MYSQLLOG")
                iter = logFilesListStore.append()
                self.logFileNotebook.append_page(self.mysqlLogFrame.get_frame(), gtk.Label(""))
                self.set_liststore_values(mysqlLogString, key, iter)
            elif key == "RPMLOG" and rpmClass.get_file_not_found() == 0:
                self.rpmLogFrame = LogFileFrame.LogFileFrame(rpmLogString, _("This log file contains a list of RPM packages installed on the system. This list is updated daily by an automated cron task."), "RPMLOG")
                iter = logFilesListStore.append()
                self.logFileNotebook.append_page(self.rpmLogFrame.get_frame(), gtk.Label(""))
                self.set_liststore_values(rpmLogString, key, iter)
            elif key == "TUXLOG" and tuxClass.get_file_not_found() == 0:
                self.tuxLogFrame = LogFileFrame.LogFileFrame(tuxLogString, _("This file records messages for the Red Hat Content Accelerator Web Server."), "TUXLOG")
                iter = logFilesListStore.append()
                self.logFileNotebook.append_page(self.tuxLogFrame.get_frame(), gtk.Label(""))
                self.set_liststore_values(tuxLogString, key, iter)
            elif key == "UP2DATELOG" and up2dateClass.get_file_not_found() == 0:
                self.up2dateLogFrame = LogFileFrame.LogFileFrame(up2dateLogString, _("The file contains a list of actions performed by the Red Hat Update Agent."), "UP2DATELOG")
                iter = logFilesListStore.append()
                self.logFileNotebook.append_page(self.up2dateLogFrame.get_frame(), gtk.Label(""))
                self.set_liststore_values(up2dateLogString, key, iter)
            elif key == "FTPLOG" and ftpClass.get_file_not_found() == 0:
                self.ftpLogFrame = LogFileFrame.LogFileFrame(ftpLogString, _("This log file contains messages from the FTP Server."), "FTPLOG")
                iter = logFilesListStore.append()
                self.logFileNotebook.append_page(self.ftpLogFrame.get_frame(), gtk.Label(""))
                self.set_liststore_values(ftpLogString, key, iter)
            elif key == "XFREE86LOG" and xfree86Class.get_file_not_found() == 0:
                self.xfree86LogFrame = LogFileFrame.LogFileFrame(xfree86LogString, _("This log file contains messages from the XFree86 server."), "XFREE86LOG")
                iter = logFilesListStore.append()
                self.logFileNotebook.append_page(self.xfree86LogFrame.get_frame(), gtk.Label(""))
                self.set_liststore_values(xfree86LogString, key, iter)
                
        col = gtk.TreeViewColumn(None, gtk.CellRendererText(), text=0)
        logFileView.set_headers_visible(gtk.FALSE)
        logFileView.append_column(col)
        logFileView.set_border_width(8)
        self.main_hpane.add1(logFileView)
        self.main_hpane.add2(self.logFileNotebook)
        selection = logFileView.get_selection()
        selection.connect("changed", self.select_category)
        iter = logFilesListStore.get_iter_root()

        #--------show gui---------------------------------------------
        self.toplevel.show_all()

        #--------select first log file in list------------------------
        selection.select_iter(iter)
        
        #--------keep track of last refresh rate pref-----------------
        self.lastRefreshRate = RefreshRate

        #--------continual updates------------------------------------
        gtk.timeout_add(RefreshRate, self.auto_refresh)

        #--------mainloop---------------------------------------------
        mainloop ()

    def set_liststore_values(self, string, key, iter):
        logFilesListStore.set_value(iter, 0, string)
        logFilesListStore.set_value(iter, 1, str(logDict[key]))
        logFilesListStore.set_value(iter, 2, key)
        logFilesListStore.set_value(iter, 3, 0)
        logFilesListStore.set_value(iter, 4, self.pgnum)
        self.pgnum = self.pgnum + 1

## About Menu Item callback

    def on_about_selected(self, action, widget):
        dlg = gtk.MessageDialog (None, 0, gtk.MESSAGE_INFO, gtk.BUTTONS_OK,
                                 _("Log Viewer 0.8.5\n Copyright (c) 2002-2003 Red Hat, Inc.\n Copyright (c) 2002-2003 Tammy Fox <tfox@redhat.com>\n A graphical interface for watching log files"))
        dlg.set_title(_("About Log Viewer"))
        dlg.set_default_size(100, 100)
        dlg.set_position (gtk.WIN_POS_CENTER_ON_PARENT)
        dlg.set_border_width(2)
        dlg.set_modal(gtk.TRUE)
        dlg.set_transient_for(self.toplevel)
        dlg.set_icon(iconPixbuf)
        rc = dlg.run()
        dlg.destroy()



## Manual Menu Item callback

    def on_manual_selected(self, action, widget):
        help_page = "file:///usr/share/doc/redhat-logviewer-" + "0.8.5" + "/index.html"
        path = "/usr/bin/htmlview"

        if path == None:
            dlg = gtk.MessageDialog(None, 0, gtk.MESSAGE_WARNING, gtk.BUTTONS_OK,
                                    (_("Help is not available.")))
            dlg.set_position(gtk.WIN_POS_CENTER_ON_PARENT)
            dlg.run()
            dlg.destroy()
            return
		
        pid = os.fork()
        if not pid:
            os.execv(path, [path, help_page])
            print "exec"

## Select Item from List Store callback

    def select_category(self, object):
        listStore, listStoreIter = object.get_selected()
        # logString is the description string from item 0 in the ListStore
        logString = listStore.get_value(listStoreIter, 0)
        # set notebook page to selected log file
        self.logFileNotebook.set_current_page(logFilesListStore.get_value(listStoreIter, 4))
        return

## Preferences Menu Item callback

    def on_preferences_selected(self, action, widget):
        prefsClass = Preferences.Preferences()
        prefsDialog = prefsClass.launch()
        prefsDialog.set_transient_for(self.toplevel)
        rc = prefsDialog.run()

        if rc == 0:
            prefsClass.on_ok_clicked()
            set_status(_("Preferences updated."))
        elif rc == 1:
            prefsClass.on_cancel_clicked()
        
## Exit Menu Item callback

    def on_exit_selected(self, action, widget):
        gtk.mainquit()

## Auto reset function
        
    def auto_refresh(self, *args):
        #get liststore iter to refresh correct page
        listStore, listStoreIter = logFileView.get_selection().get_selected()
        # get current page number
        pageNum = logFilesListStore.get_value(listStoreIter, 4)
        logClassInstance = logFilesListStore.get_value(listStoreIter, 2)

        #restart timeout function if refresh rate pref has changed
        if RefreshRate != self.lastRefreshRate:
            gtk.timeout_add(self.lastRefreshRate, self.auto_refresh)
            self.lastRefreshRate = RefreshRate            
            return gtk.FALSE

        set_status(_("Refreshing log file..."))

        #call refresh if file changed
        if logClassInstance == "SYSLOG" and sysClass.fileChanged():
            sysClass.auto_refresh()
            self.systemLogFrame.set_buffer(sysClass.buffer.get_buffer())
        elif logClassInstance == "SECURELOG" and secureClass.fileChanged():
            secureClass.auto_refresh()
            self.secureLogFrame.set_buffer(secureClass.buffer.get_buffer())
        elif logClassInstance == "MAILLOG" and mailClass.fileChanged():
            mailClass.auto_refresh()
            self.mailLogFrame.set_buffer(mailClass.buffer.get_buffer())
        elif logClassInstance == "CRONLOG" and cronClass.fileChanged():
            cronClass.auto_refresh()
            self.cronLogFrame.set_buffer(cronClass.buffer.get_buffer())
        elif logClassInstance == "NEWSLOG" and newsClass.fileChanged():
            newsClass.auto_refresh()
            self.newsLogFrame.set_buffer(newsClass.buffer.get_buffer())
        elif logClassInstance == "BOOTLOG" and bootClass.fileChanged():
            bootClass.auto_refresh()
            self.bootLogFrame.set_buffer(bootClass.buffer.get_buffer())
        elif logClassInstance == "CIPELOG" and cipeClass.fileChanged():
            cipeClass.auto_refresh()
            self.cipeLogFrame.set_buffer(cipeClass.buffer.get_buffer())
        elif logClassInstance == "DMESGLOG" and dmesgClass.fileChanged():
            dmesgClass.auto_refresh()
            self.dmesgLogFrame.set_buffer(dmesgClass.buffer.get_buffer())
        elif logClassInstance == "HTTPDACCESSLOG" and httpdaccessClass.fileChanged():
            httpdaccessClass.auto_refresh()
            self.httpdaccessLogFrame.set_buffer(httpdaccessClass.buffer.get_buffer())
        elif logClassInstance == "HTTPDERROR" and httpderrorClass.fileChanged():
            httpderrorClass.auto_refresh()
            self.httpderrorLogFrame.set_buffer(httpderrorClass.buffer.get_buffer())
        elif logClassInstance == "MYSQLLOG" and mysqlClass.fileChanged():
            mysqlClass.auto_refresh()
            self.mysqlLogFrame.set_buffer(mysqlClass.buffer.get_buffer())
        elif logClassInstance == "RPMLOG" and rpmClass.fileChanged():
            rpmClass.auto_refresh()
            self.rpmLogFrame.set_buffer(rpmClass.buffer.get_buffer())
        elif logClassInstance == "TUXLOG" and tuxClass.fileChanged():
            tuxClass.auto_refresh()
            self.tuxLogFrame.set_buffer(tuxClass.buffer.get_buffer())
        elif logClassInstance == "UP2DATELOG" and up2dateClass.fileChanged():
            up2dateClass.auto_refresh()
            self.up2dateLogFrame.set_buffer(up2dateClass.buffer.get_buffer())
        elif logClassInstance == "FTPLOG" and ftpClass.fileChanged():
            ftpClass.auto_refresh()
            self.ftpLogFrame.set_buffer(ftpClass.buffer.get_buffer())
        elif logClassInstance == "XFREE86LOG" and xfree86Class.fileChanged():
            xfree86Class.auto_refresh()
            self.xfree86LogFrame.set_buffer(xfree86Class.buffer.get_buffer())

        #reset status bar after auto_refresh
        set_status("")

        #keep track if refreshrate changes
        self.lastRefreshRate = RefreshRate
        #return TRUE so gtk.time_out will keep going
        return gtk.TRUE
    
